@@ -24,13 +24,12 @@
 #' @param rug TRUE displays tick marks on the axes to mark the distribution of raw data.
 #' @param gray FALSE grayscale or color plot
 #' @param draw `TRUE` returns a `ggplot2` plot. `FALSE` returns a `data.frame` of the underlying data.
-#' @param newdata When `newdata` is `NULL`, the grid is determined by the `condition` argument. When `newdata` is not `NULL`, the argument behaves in the same way as in the `slopes()` function.
+#' @param newdata When `newdata` is `NULL`, the grid is determined by the `condition` argument. When `newdata` is not `NULL`, the argument behaves in the same way as in the `predictions()` function. Note that the `condition` argument builds its own grid, so the `newdata` argument is ignored if the `condition` argument is supplied.
 #' @inheritParams slopes
 #' @template model_specific_arguments
 #' @return A `ggplot2` object
 #' @export
 #' @examplesIf interactive() || isTRUE(Sys.getenv("R_DOC_BUILD") == "true")
-#' library(marginaleffects)
 #' mod <- lm(mpg ~ hp * drat * factor(am), data = mtcars)
 #'
 #' plot_slopes(mod, variables = "hp", condition = "drat")
@@ -48,68 +47,83 @@
 #'
 #' # marginal slopes on a counterfactual grid
 #' plot_slopes(mod,
-#'   variables = "hp",
-#'   by = "am",
-#'   newdata = datagrid(am = 0:1, grid_type = "counterfactual")
+#'     variables = "hp",
+#'     by = "am",
+#'     newdata = datagrid(am = 0:1, grid_type = "counterfactual")
 #' )
 #'
-plot_slopes <- function(model,
-                        variables = NULL,
-                        condition = NULL,
-                        by = NULL,
-                        newdata = NULL,
-                        type = NULL,
-                        vcov = NULL,
-                        conf_level = 0.95,
-                        wts = FALSE,
-                        slope = "dydx",
-                        rug = FALSE,
-                        gray = getOption("marginaleffects_plot_gray", default = FALSE),
-                        draw = TRUE,
-                        ...) {
-  dots <- list(...)
-  if ("effect" %in% names(dots)) {
-    if (is.null(variables)) {
-      variables <- dots[["effect"]]
-    } else {
-      insight::format_error("The `effect` argument has been renamed to `variables`.")
-    }
-  }
-
-  if (inherits(model, "mira") && is.null(newdata)) {
-    msg <- "Please supply a data frame to the `newdata` argument explicitly."
-    insight::format_error(msg)
-  }
-
-  # order of the first few paragraphs is important
-  # if `newdata` is a call to `typical` or `counterfactual`, insert `model`
-  # should probably not be nested too deeply in the call stack since we eval.parent() (not sure about this)
-  scall <- rlang::enquo(newdata)
-  newdata <- sanitize_newdata_call(scall, newdata, model)
-
-  valid <- c("dydx", "eyex", "eydx", "dyex")
-  checkmate::assert_choice(slope, choices = valid)
-
-  out <- plot_comparisons(
+plot_slopes <- function(
     model,
-    variables = variables,
-    condition = condition,
-    by = by,
-    newdata = newdata,
-    type = type,
-    vcov = vcov,
-    conf_level = conf_level,
-    wts = wts,
-    draw = draw,
-    rug = rug,
-    gray = gray,
-    comparison = slope,
-    ...
-  )
+    variables = NULL,
+    condition = NULL,
+    by = NULL,
+    newdata = NULL,
+    type = NULL,
+    vcov = NULL,
+    conf_level = 0.95,
+    wts = FALSE,
+    slope = "dydx",
+    rug = FALSE,
+    gray = getOption("marginaleffects_plot_gray", default = FALSE),
+    draw = TRUE,
+    ...) {
+    if ("effect" %in% ...names()) {
+        if (is.null(variables)) {
+            variables <- ...elt(match("effect", ...names())[1L])
+        } else {
+            stop_sprintf(
+                "The `effect` argument has been renamed to `variables`."
+            )
+        }
+    }
 
-  if (inherits(out, "ggplot")) {
-    out <- out + ggplot2::labs(x = condition[1], y = "Slope")
-  }
+    if (inherits(model, c("mira", "amest"))) {
+        msg <- "This function does not support multiple imputation. Call `slopes()` or `avg_slopes()` instead. These functions return easy to plot data frames."
+        stop_sprintf(msg)
+    }
 
-  return(out)
+    # init
+    call <- construct_call(model, "comparisons")
+    model <- sanitize_model(model, call = call, newdata = newdata, wts = wts, vcov = vcov, by = by, ...)
+    mfx <- new_marginaleffects_internal(
+        call = call,
+        model = model
+    )
+
+    if (inherits(mfx@model, "mira") && is.null(newdata)) {
+        msg <- "Please supply a data frame to the `newdata` argument explicitly."
+        stop_sprintf(msg)
+    }
+
+    # order of the first few paragraphs is important
+    # if `newdata` is a call to `typical` or `counterfactual`, insert `model`
+    # should probably not be nested too deeply in the call stack since we eval.parent() (not sure about this)
+    scall <- rlang::enquo(newdata)
+    newdata <- sanitize_newdata_call(scall, newdata, mfx = mfx)
+
+    valid <- c("dydx", "eyex", "eydx", "dyex")
+    checkmate::assert_choice(slope, choices = valid)
+
+    out <- plot_comparisons(
+        mfx@model,
+        variables = variables,
+        condition = condition,
+        by = by,
+        newdata = newdata,
+        type = type,
+        vcov = vcov,
+        conf_level = conf_level,
+        wts = wts,
+        draw = draw,
+        rug = rug,
+        gray = gray,
+        comparison = slope,
+        ...
+    )
+
+    if (inherits(out, "ggplot")) {
+        attr(out$data$estimate, "label") <- "Slope"
+    }
+
+    return(out)
 }
