@@ -13,6 +13,8 @@
 #' * [https://marginaleffects.com/chapters/slopes.html](https://marginaleffects.com/chapters/slopes.html)
 #' * [https://marginaleffects.com/](https://marginaleffects.com/)
 #'
+#' Warning: Slopes and elasticities can only be calculated for continuous numeric variables. The `slopes()` functions will automatically revert to `comparisons()` for binary or categorical variables.
+#'
 #' @details
 #' A "slope" or "marginal effect" is the partial derivative of the regression equation
 #' with respect to a variable in the model. This function uses automatic
@@ -46,6 +48,8 @@
 #'    - Heteroskedasticity and autocorrelation consistent: `"HAC"`
 #'    - Mixed-Models degrees of freedom: "satterthwaite", "kenward-roger"
 #'    - Other: `"NeweyWest"`, `"KernHAC"`, `"OPG"`. See the `sandwich` package documentation.
+#'    - "rsample", "boot", "fwb", and "simulation" are passed to the `method` argument of the `inferences()` function. To customize the bootstrap or simulation process, call `inferences()` directly.
+#'
 #'  * One-sided formula which indicates the name of cluster variables (e.g., `~unit_id`). This formula is passed to the `cluster` argument of the `sandwich::vcovCL` function.
 #'  * Square covariance matrix
 #'  * Function which returns a covariance matrix (e.g., `stats::vcov(model)`)
@@ -55,7 +59,7 @@
 #' type, but will typically be a string such as: "response", "link", "probs",
 #' or "zero". When an unsupported string is entered, the model-specific list of
 #' acceptable values is returned in an error message. When `type` is `NULL`, the
-#' first entry in the error message is used by default.
+#' first entry in the error message is used by default. See the Type section in the documentation below.
 #' @param slope string indicates the type of slope or (semi-)elasticity to compute:
 #' - "dydx": dY/dX
 #' - "eyex": dY/dX * Y / X
@@ -69,22 +73,25 @@
 #' + TRUE: Extract weights from the fitted object with `insight::find_weights()` and use them when taking weighted averages of estimates. Warning: `newdata=datagrid()` returns a single average weight, which is equivalent to using `wts=FALSE`
 #' @param hypothesis specify a hypothesis test or custom contrast using a number , formula, string equation, vector, matrix, or function.
 #' + Number: The null hypothesis used in the computation of Z and p (before applying `transform`).
-#' + String: Equation to specify linear or non-linear hypothesis tests. If the terms in `coef(object)` uniquely identify estimates, they can be used in the formula. Otherwise, use `b1`, `b2`, etc. to identify the position of each parameter. The `b*` wildcard can be used to test hypotheses on all estimates. If a named vector is used, the names are used as labels in the output. Examples:
+#' + String: Equation to specify linear or non-linear hypothesis tests. Two-tailed tests must include an equal `=` sign. One-tailed tests must start with `<` or `>`. If the terms in `coef(object)` uniquely identify estimates, they can be used in the formula. Otherwise, use `b1`, `b2`, etc. to identify the position of each parameter. The `b*` wildcard can be used to test hypotheses on all estimates. When the hypothesis string represents a two-sided equation, the `estimate` column holds the value of the left side minus the right side of the equation. If a named vector is used, the names are used as labels in the output. Examples:
 #'   - `hp = drat`
 #'   - `hp + drat = 12`
 #'   - `b1 + b2 + b3 = 0`
 #'   - `b* / b1 = 1`
+#'   - `<= 0`
+#'   - `>= -3.5`
+#'   - `b1 >= 10`
 #' + Formula: `lhs ~ rhs | group`
 #'   + `lhs`
-#'     - `ratio`
-#'     - `difference`
+#'     - `ratio` (null = 1)
+#'     - `difference` (null = 0)
 #'     - Leave empty for default value
 #'   + `rhs`
 #'     - `pairwise` and `revpairwise`: pairwise differences between estimates in each row.
 #'     - `reference`: differences between the estimates in each row and the estimate in the first row.
 #'     - `sequential`: difference between an estimate and the estimate in the next row.
 #'     - `meandev`: difference between an estimate and the mean of all estimates.
-#'     - `meanotherdev: difference between an estimate and the mean of all other estimates, excluding the current one.
+#'     - `meanotherdev`: difference between an estimate and the mean of all other estimates, excluding the current one.
 #'     - `poly`: polynomial contrasts, as computed by the `stats::contr.poly()` function.
 #'     - `helmert`: Helmert contrasts, as computed by the `stats::contr.helmert()` function. Contrast 2nd level to the first, 3rd to the average of the first two, and so on.
 #'     - `trt_vs_ctrl`: difference between the mean of estimates (except the first) and the first estimate.
@@ -106,7 +113,11 @@
 #'   - The function can also accept optional input arguments: `newdata`, `by`, `draws`.
 #'   - This function approach will not work for Bayesian models or with bootstrapping. In those cases, it is easy to use `get_draws()` to extract and manipulate the draws directly.
 #' + See the Examples section below and the vignette: [https://marginaleffects.com/chapters/hypothesis.html](https://marginaleffects.com/chapters/hypothesis.html)
-#' @param df Degrees of freedom used to compute p values and confidence intervals. A single numeric value between 1 and `Inf`. When `df` is `Inf`, the normal distribution is used. When `df` is finite, the `t` distribution is used. See [insight::get_df] for a convenient function to extract degrees of freedom. Ex: `slopes(model, df = insight::get_df(model))`
+#' + Warning: When calling `predictions()` with `type="invlink(link)"` (the default in some models), `hypothesis` is tested and p values are computed on the link scale.
+#' @param df Degrees of freedom used to compute p values and confidence intervals.
+#'   - A single numeric value between 1 and `Inf`, or a numeric vector with length equal to the number of rows in the output. When `df` is `Inf`, the normal distribution is used. When `df` is finite, the `t` distribution is used.
+#'   - "residual": Calls [insight::get_df] to extract degrees of freedom from the model automatically.
+#'   - "satterthwaite" or "kenward-roger": Use the Satterthwaite or Kenward-Roger approximation to compute degrees of freedom in mixed effects models.
 #' @param eps NULL or numeric value which determines the step size to use when
 #' calculating numerical derivatives: (f(x+eps)-f(x))/eps. When `eps` is
 #' `NULL`, the step size is 0.0001 multiplied by the difference between
@@ -114,8 +125,8 @@
 #' are taking the derivative. Changing `eps` may be necessary to avoid
 #' numerical problems in certain models.
 #' @param numderiv string or list of strings indicating the method to use to for the numeric differentiation used in to compute delta method standard errors.
-#' + "fdforward": finite difference method with forward differences
-#' + "fdcenter": finite difference method with central differences (default)
+#' + "fdforward": finite difference method with forward differences (default)
+#' + "fdcenter": finite difference method with central differences
 #' + "richardson": Richardson extrapolation method
 #' + Extra arguments can be specified by passing a list to the `numDeriv` argument, with the name of the method first and named arguments following, ex: `numderiv=list("fdcenter", eps = 1e-5)`. When an unknown argument is used, `marginaleffects` prints the list of valid arguments for each method.
 #' @param ... Additional arguments are passed to the `predict()` method
@@ -138,12 +149,13 @@
 #' @template model_specific_arguments
 #' @template bayesian
 #' @template equivalence
-#' @template type
 #' @template parallel
 #' @template order_of_operations
 #' @template options
 #' @template return
+#' @template type
 #' @examplesIf interactive() || isTRUE(Sys.getenv("R_DOC_BUILD") == "true")
+#' library("marginaleffects")
 #' # Unit-level (conditional) Marginal Effects
 #' mod <- glm(am ~ hp * wt, data = mtcars, family = binomial)
 #' mfx <- slopes(mod)
@@ -172,9 +184,9 @@
 #' # original values, and the whole dataset is duplicated once for each
 #' # combination of the values in `datagrid()`
 #' mfx <- slopes(mod,
-#'   newdata = datagrid(
-#'     hp = c(100, 110),
-#'     grid_type = "counterfactual"))
+#'     newdata = datagrid(
+#'         hp = c(100, 110),
+#'         grid_type = "counterfactual"))
 #' head(mfx)
 #'
 #' # Heteroskedasticity robust standard errors
@@ -185,184 +197,115 @@
 #' mod <- lm(mpg ~ wt + drat, data = mtcars)
 #'
 #' slopes(
-#'   mod,
-#'   newdata = "mean",
-#'   hypothesis = "wt = drat")
+#'     mod,
+#'     newdata = "mean",
+#'     hypothesis = "wt = drat")
 #'
 #' # same hypothesis test using row indices
 #' slopes(
-#'   mod,
-#'   newdata = "mean",
-#'   hypothesis = "b1 - b2 = 0")
+#'     mod,
+#'     newdata = "mean",
+#'     hypothesis = "b1 - b2 = 0")
 #'
 #' # same hypothesis test using numeric vector of weights
 #' slopes(
-#'   mod,
-#'   newdata = "mean",
-#'   hypothesis = c(1, -1))
+#'     mod,
+#'     newdata = "mean",
+#'     hypothesis = c(1, -1))
 #'
 #' # two custom contrasts using a matrix of weights
 #' lc <- matrix(
-#'   c(
-#'     1, -1,
-#'     2, 3),
-#'   ncol = 2)
+#'     c(
+#'         1, -1,
+#'         2, 3),
+#'     ncol = 2)
 #' colnames(lc) <- c("Contrast A", "Contrast B")
 #' slopes(
-#'   mod,
-#'   newdata = "mean",
-#'   hypothesis = lc)
+#'     mod,
+#'     newdata = "mean",
+#'     hypothesis = lc)
 #'
 #' @export
-slopes <- function(model,
-                   newdata = NULL,
-                   variables = NULL,
-                   type = NULL,
-                   by = FALSE,
-                   vcov = TRUE,
-                   conf_level = 0.95,
-                   slope = "dydx",
-                   wts = FALSE,
-                   hypothesis = NULL,
-                   equivalence = NULL,
-                   df = Inf,
-                   eps = NULL,
-                   numderiv = "fdforward",
-                   ...) {
-  dots <- list(...)
-
-  # very early, before any use of newdata
-  # if `newdata` is a call to `typical` or `counterfactual`, insert `model`
-  scall <- rlang::enquo(newdata)
-  newdata <- sanitize_newdata_call(scall, newdata, model, by = by)
-
-  # build call: match.call() doesn't work well in *apply()
-  call_attr <- c(
-    list(
-      name = "slopes",
-      model = model,
-      newdata = newdata,
-      variables = variables,
-      type = type,
-      vcov = vcov,
-      by = by,
-      conf_level = conf_level,
-      slope = slope,
-      wts = wts,
-      hypothesis = hypothesis,
-      df = df,
-      eps = eps),
-    list(...))
-  call_attr <- do.call("call", call_attr)
-
-  # slopes() does not support a named list of variables like comparisons()
-  checkmate::assert_character(variables, null.ok = TRUE)
-
-  # slope
-  valid <- c("dydx", "eyex", "eydx", "dyex", "dydxavg", "eyexavg", "eydxavg", "dyexavg")
-  checkmate::assert_choice(slope, choices = valid)
-
-  # sanity checks and pre-processing
-  model <- sanitize_model(model = model, newdata = newdata, wts = wts, vcov = vcov, by = by, calling_function = "marginaleffects", ...)
-  sanity_dots(model = model, calling_function = "marginaleffects", ...)
-  type <- sanitize_type(model = model, type = type, calling_function = "slopes")
-
-  ############### sanity checks are over
-
-  # Bootstrap
-  out <- inferences_dispatch(
-    INF_FUN = slopes,
-    model = model, newdata = newdata, vcov = vcov, variables = variables, type = type,
-    conf_level = conf_level,
-    by = by,
-    wts = wts, slope = slope, hypothesis = hypothesis, ...)
-  if (!is.null(out)) {
-    return(out)
-  }
-
-  out <- comparisons(
+slopes <- function(
     model,
-    newdata = newdata,
-    variables = variables,
-    vcov = vcov,
-    conf_level = conf_level,
-    type = type,
-    wts = wts,
-    hypothesis = hypothesis,
-    equivalence = equivalence,
-    df = df,
-    by = by,
-    eps = eps,
-    numderiv = numderiv,
-    comparison = slope,
-    cross = FALSE,
-    # secret arguments
-    internal_call = TRUE,
-    ...)
+    newdata = NULL,
+    variables = NULL,
+    type = NULL,
+    by = FALSE,
+    vcov = TRUE,
+    conf_level = 0.95,
+    slope = "dydx",
+    wts = FALSE,
+    hypothesis = NULL,
+    equivalence = NULL,
+    df = Inf,
+    eps = NULL,
+    numderiv = "fdforward",
+    ...) {
+    call_attr <- construct_call(model, "slopes")
 
-  # class
-  data.table::setDF(out)
-  class(out) <- setdiff(class(out), "comparisons")
-  class(out) <- c("slopes", "marginaleffects", class(out))
-  return(out)
+    # slopes() does not support a named list of variables like comparisons()
+    checkmate::assert_character(variables, null.ok = TRUE)
+
+    # slope
+    valid <- c(
+        "dydx",
+        "eyex",
+        "eydx",
+        "dyex",
+        "dydxavg",
+        "eyexavg",
+        "eydxavg",
+        "dyexavg"
+    )
+    checkmate::assert_choice(slope, choices = valid)
+
+    # sanity checks and pre-processing
+    # Note: model sanitization is handled in comparisons()
+    sanity_dots(model = model, calling_function = "slopes", ...)
+
+    ############### sanity checks are over
+
+    call_attr_c <- call_attr
+    call_attr_c[[1L]] <- quote(marginaleffects::comparisons)
+    call_attr_c[["model"]] <- model
+    call_attr_c[["type"]] <- type
+    call_attr_c[["comparison"]] <- slope
+    call_attr_c[["cross"]] <- FALSE
+    call_attr_c[["internal_call"]] <- TRUE
+    call_attr_c[["slope"]] <- NULL
+
+    out <- eval.parent(call_attr_c)
+
+    # class
+    data.table::setDF(out)
+    class(out) <- setdiff(class(out), "comparisons")
+    class(out) <- c("slopes", "marginaleffects", class(out))
+    return(out)
 }
 
 
-
-
 #' Average slopes (aka Average partial derivatives, marginal effects, or trends)
 #' @describeIn slopes Average slopes
 #' @export
 #'
-avg_slopes <- function(model,
-                       newdata = NULL,
-                       variables = NULL,
-                       type = NULL,
-                       by = TRUE,
-                       vcov = TRUE,
-                       conf_level = 0.95,
-                       slope = "dydx",
-                       wts = FALSE,
-                       hypothesis = NULL,
-                       equivalence = NULL,
-                       df = Inf,
-                       eps = NULL,
-                       numderiv = "fdforward",
-                       ...) {
-  # order of the first few paragraphs is important
-  # if `newdata` is a call to `typical` or `counterfactual`, insert `model`
-  # should probably not be nested too deeply in the call stack since we eval.parent() (not sure about this)
-  scall <- rlang::enquo(newdata)
-  newdata <- sanitize_newdata_call(scall, newdata, model, by = by)
-
-
-  # Bootstrap
-  out <- inferences_dispatch(
-    INF_FUN = avg_slopes,
-    model = model, newdata = newdata, vcov = vcov, variables = variables, type = type,
-    conf_level = conf_level, by = by,
-    wts = wts, slope = slope, hypothesis = hypothesis, ...)
-  if (!is.null(out)) {
+avg_slopes <- function(
+    model,
+    newdata = NULL,
+    variables = NULL,
+    type = NULL,
+    by = TRUE,
+    vcov = TRUE,
+    conf_level = 0.95,
+    slope = "dydx",
+    wts = FALSE,
+    hypothesis = NULL,
+    equivalence = NULL,
+    df = Inf,
+    eps = NULL,
+    numderiv = "fdforward",
+    ...) {
+    call_attr <- construct_call(model, "slopes")
+    out <- eval.parent(call_attr)
     return(out)
-  }
-
-
-  out <- slopes(
-    model = model,
-    newdata = newdata,
-    variables = variables,
-    type = type,
-    vcov = vcov,
-    conf_level = conf_level,
-    by = by,
-    slope = slope,
-    wts = wts,
-    hypothesis = hypothesis,
-    equivalence = equivalence,
-    df = df,
-    eps = eps,
-    numderiv = numderiv,
-    ...)
-
-  return(out)
 }
